/*
 -------------------------------------------------------------------------
 Copyright (c) 2002, Markus Lagler <markus.lagler@trivadis.com>
 Copyright (c) 2002, Tim Tassonis  <timtas@dplanet.ch>

 All rights reserved.
 
 LICENSE TERMS
 
 The free distribution and use of this software in both source and binary
 form is allowed (with or without changes) provided that:
   
   1. distributions of this source code include the above copyright
      notice, this list of conditions and the following disclaimer;
   
   2. distributions in binary form include the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other associated materials;
   
   3. the copyright holder's name is not used to endorse products
      built using this software without specific written permission.
 
 DISCLAIMER
 
 This software is provided 'as is' with no explcit or implied warranties
 in respect of any properties, including, but not limited to, correctness
 and fitness for purpose.
 -------------------------------------------------------------------------
*/
#include <stdio.h>
#include <errno.h>
#include <string.h>
#ifndef WINDOWS
#include <unistd.h>
#endif
#include <time.h>

#include "aeslib.h"
#include "lutil.h"
#include "getopts.h"

#define USAGE_STR "\nUsage: %s [-d|-e] [-p password] [-b] [-i data]\n\t-d\tdecrypt\n\t-e\tencrypt\n\t-p\tpassword\n\t-b\tuse base 64 encoding\n\t-i\tuse this data instead of stdin\n"

char *read_stdin(int *buffer_bytes);


#ifdef WINDOWS
/* Quick and dirty password check, maximum length is 255 bytes */
#include <conio.h>
char * getpass(const char *prompt)
{
	int eol,j;
	signed int byte;
	static char password[255];
	eol = 0;
	printf(prompt);
	for (j=0 ; eol == 0; j++) {
		byte = getch();
		if (byte == 13 ) {
			eol=1;
			password[j] = '\0';
			fflush(stdin);
			printf("\n");
		} else {
			printf("*");
			password[j] = byte;
		}
	}
	return password;
}
#endif

int main(int argc, char **argv)
{
	int rc=0,opt,i;
	char *args;
	int argsize=0;
	char *enc_buffer;
	char *dec_buffer;
	char *b64_buffer;
	char *in_buffer;
	char password[17] = "\0";
	int enc_len=0,dec_len=0,in_len=0;
	int base64=0,encrypt=0,decrypt=0;
	struct options opts[] =
	{
		{ 1, "encrypt",   "Encrypt the data",  "e", 0 },
		{ 2, "decrypt",   "Decrypt the data",  "d", 0 },
		{ 3, "base64",    "base64 encode/decode encrypted data",  "b", 0 },
		{ 4, "password",  "Password (if omitted, will prompt)",  "p", 1 },
		{ 5, "input",     "Data (if omitted, stdin)", "i", 1 },
		{ 0, NULL,      NULL,                      NULL, 0 }
	};
	/* First, lets allocate enough space for the arguments */
	argsize=0;
	for (i=0; i< argc; i++) {
		argsize += strlen(argv[i])+1;
	}
	args = malloc(argsize);
	memset(args, 0, argsize);
	if (args == NULL) {
		fprintf(stderr,"Failed to allocate %d bytes for args\n",argsize);
		exit(1);
	}
	while ( (opt = getopts(argc, argv, opts, args)) != 0) {
		switch(opt) {
			case 1:
				encrypt=1;
				break;
			case 2:
				decrypt=1;
				break;
			case 3:
				base64=1;
				break;
			case 4:
				strncpy(password,args,16);
				args[16] = '\0';
				break;
			case 5:
				in_len = strlen(args);
				in_buffer = malloc(in_len+1);
				memcpy(in_buffer,args,in_len);
				break;
			default:
				break;
		}
		memset(args, 0, argsize);
	}
	if ( (encrypt+decrypt) > 1 ) {
		fprintf(stderr,"Only one of -d/-e may be specified!\n");
		fprintf(stderr,USAGE_STR,argv[0]);
		exit(1);
	}
	if ( (encrypt+decrypt) < 1 ) {
		fprintf(stderr,"One of -d/-e must be specified!\n");
		fprintf(stderr,USAGE_STR,argv[0]);
		exit(1);
	}
	if (strlen(password)  < 1 ) {
		strncpy(password,getpass("Enter password :"),16);
		password[16] = '\0';
	}
	if (strlen(password)  < 6 ) {
		fprintf(stderr,"A password of at least 6 bytes must be specified!\n");
		fprintf(stderr,USAGE_STR,argv[0]);
		exit(1);
	}
	if (in_len == 0) {
		in_buffer = read_stdin(&in_len);
	}
	if (in_len < 1) {
		fprintf(stderr,"No input specified!\n");
		fprintf(stderr,USAGE_STR,argv[0]);
		exit(1);
	}
	if (encrypt) {
		enc_buffer = aes_enc(in_buffer, in_len, password, strlen(password), &enc_len);
		if (enc_buffer == NULL) {
			fprintf(stderr,"Encryption failed!\n");
			exit(1);
		}
		if (base64) {
			b64_buffer = malloc(enc_len*2);
			rc = lutil_b64_ntop(enc_buffer,enc_len,b64_buffer,enc_len*2);
			fwrite(b64_buffer,1,rc,stdout);
			printf("\n");
		} else {
			fwrite(enc_buffer,1,enc_len,stdout);
		}
	} else if (decrypt) {
		if (base64) {
			enc_buffer = malloc(in_len);
			rc = lutil_b64_pton(in_buffer,enc_buffer,in_len);
			enc_len=rc;
		} else {
			enc_buffer = in_buffer;
			enc_len    = in_len;
		}
		dec_buffer = aes_dec(enc_buffer,enc_len,password,strlen(password),&dec_len);
		if (dec_buffer == NULL) {
			fprintf(stderr,"Decryption failed!\n");
			exit(1);
		}
		fwrite(dec_buffer,1,dec_len,stdout);
	}
	return 0;
}



char *read_stdin(int *buffer_bytes)
{

#define MSG_BLOCK_SIZE 65025
	static char *msgbuf;
	char *tmp_buffer;
	int tempfile;
	int temp_open=0;
	FILE *tfile;
#ifdef WINDOWS
	char *tempfilename;
#else
	char tempfilename[2000] = "/tmp/aes_inputXXXXXX" ;
#endif
	int rc;
	int i;
	int bytes,tbytes=0,write_bytes;
	int msglen;
	int offset;
#ifdef WINDOWS
	tempfilename = tempnam(NULL,"aes");
	tfile = fopen(tempfilename,"wb");
	if (tfile == NULL ) {
		return NULL;
	}
	temp_open = 1;
#else
	tempfile = mkstemp(tempfilename);
	if (tempfile == -1) {
		return NULL;
	}
	tfile = fdopen(tempfile,"r+");
	if (tfile == NULL ) {
		close(tempfile);
		rc = -2; goto exit;
	}
	temp_open = 1;
#endif
	msglen = MSG_BLOCK_SIZE;
	offset = 0;
	tmp_buffer = malloc(MSG_BLOCK_SIZE);
	if (tmp_buffer == NULL) {
		rc= -3; goto exit;
	}
	i=0;
	while (!feof(stdin)) {
		bytes = fread(tmp_buffer,1,msglen,stdin);
		if (bytes > 0 ) {
			offset += bytes;
			write_bytes = fwrite(tmp_buffer,1,bytes,tfile);
			if ( write_bytes == -1 ) {
				rc = -4; goto exit;
			}
			tbytes += bytes;
		}
	}
	free(tmp_buffer);
	fclose(tfile);
	temp_open = 0;
	msglen = tbytes;
	msgbuf = malloc(msglen);
	if (msgbuf == NULL ) {
		rc = -5; goto exit;
	}
#ifdef WINDOWS
	tfile = fopen(tempfilename,"rb");
#else
	tfile = fopen(tempfilename,"r");
#endif
	if (tfile == NULL ) {
		rc = -6; goto exit;
	}
	temp_open = 1;
	bytes = fread(msgbuf,1,msglen,tfile);
	if (bytes != msglen) {
		rc = -7; goto exit;
	}
	rc = bytes; goto exit;
exit:
	if (temp_open) {
		fclose(tfile);
	}
	remove(tempfilename);
	*buffer_bytes = rc;
	if (rc >0 ) {
		return msgbuf;
	} else {
		return NULL;
	}
}
